//+------------------------------------------------------------------+
//|                                                  FxCraftRSL.java |
//|                                   ccc   FxCraft :: Jan Jablonski |
//|                                           Copyright 2013 FxCraft |
//|                                 licencja: Creative Commons BY-SA |
//|                                              http://fxcraft.biz/ |
//+------------------------------------------------------------------+
package jforex;
import com.dukascopy.api.*;
import com.dukascopy.api.indicators.*;
import java.awt.*;

public class FxCraftRSL implements IIndicator {
    private IndicatorInfo indicatorInfo;
    private InputParameterInfo[] inputParameterInfos;
    private OptInputParameterInfo[] optInputParameterInfos;
    private OutputParameterInfo[] outputParameterInfos;
    private double[][] inputs = new double[1][];
    private int timePeriod = 27;
    private double[][] outputs = new double[1][];
    private IIndicator smaIndicator;
    IConsole consol;
    
    public void onStart(IIndicatorContext context) {
        this.consol = context.getConsole();
        smaIndicator = context.getIndicatorsProvider().getIndicator("SMA");
        indicatorInfo = new IndicatorInfo("RSL", "Relative Strength Index Levy", "fxcraft.biz",
                false, false, false, 1, 1, 1);
        inputParameterInfos = new InputParameterInfo[] {
            new InputParameterInfo("Price", InputParameterInfo.Type.DOUBLE){{
              setAppliedPrice(IIndicators.AppliedPrice.CLOSE);  
            }}
        };
        optInputParameterInfos = new OptInputParameterInfo[] {
            new OptInputParameterInfo("Time period", OptInputParameterInfo.Type.OTHER, new IntegerRangeDescription(27, 2, 500, 1))
            };
        outputParameterInfos = new OutputParameterInfo[] {
            new OutputParameterInfo("RSL = Price/SMA(Time Period)", OutputParameterInfo.Type.DOUBLE, OutputParameterInfo.DrawingStyle.LINE)
            };
    }

    public IndicatorResult calculate(int startIndex, int endIndex) {
        if (startIndex - getLookback() < 0) {
            startIndex -= startIndex - getLookback();
        }
        if (startIndex > endIndex) {
            return new IndicatorResult(0, 0);
        }
        int i, j;
        for (i = startIndex, j = 0; i <= endIndex; i++, j++) {
            //calculating sma
            double[] valueSMA = new double[endIndex - startIndex + getLookback()+1];
            smaIndicator.setInputParameter(0, inputs[0]);
            smaIndicator.setOptInputParameter(0, timePeriod);
            smaIndicator.setOutputParameter(0, valueSMA);
            IndicatorResult smaResult = smaIndicator.calculate(startIndex, endIndex);
            double value = 0;
            value = (inputs[0][i]/valueSMA[i-getLookback()]);
            outputs[0][j] = value;
        }
        return new IndicatorResult(startIndex, j);
    }

    public IndicatorInfo getIndicatorInfo() {
        return indicatorInfo;
    }

    public InputParameterInfo getInputParameterInfo(int index) {
        if (index <= inputParameterInfos.length) {
            return inputParameterInfos[index];
        }
        return null;
    }

    public int getLookback() {
        return timePeriod;
    }

    public int getLookforward() {
        return 0;
    }

    public OptInputParameterInfo getOptInputParameterInfo(int index) {
        if (index <= optInputParameterInfos.length) {
            return optInputParameterInfos[index];
        }
        return null;
    }

    public OutputParameterInfo getOutputParameterInfo(int index) {
        if (index <= outputParameterInfos.length) {
            return outputParameterInfos[index];
        }
        return null;
    }

    public void setInputParameter(int index, Object array) {
        inputs[index] = (double[]) array;
    }

    public void setOptInputParameter(int index, Object value) {
        timePeriod = (Integer) value;
    }

    public void setOutputParameter(int index, Object array) {
        outputs[index] = (double[]) array;
    }
}